@@ -218,6 +218,7 @@ _g_ir_node_free (GIrNode *node)
 
 	g_free (node->name);
 	g_free (function->symbol);
+        g_free (function->property);
 	_g_ir_node_free ((GIrNode *)function->result);
 	for (l = function->parameters; l; l = l->next)
 	  _g_ir_node_free ((GIrNode *)l->data);
@@ -253,6 +254,8 @@ _g_ir_node_free (GIrNode *node)
 	GIrNodeProperty *property = (GIrNodeProperty *)node;
 
 	g_free (node->name);
+        g_free (property->setter);
+        g_free (property->getter);
 	_g_ir_node_free ((GIrNode *)property->type);
       }
       break;
@@ -1626,6 +1629,36 @@ _g_ir_node_build_typelib (GIrNode         *node,
 	blob->transfer_container_ownership = prop->shallow_transfer;
 	blob->reserved = 0;
 
+        if (prop->setter != NULL)
+          {
+            int index = get_index_of_member_type ((GIrNodeInterface*)parent,
+                                                  G_IR_NODE_FUNCTION,
+                                                  prop->setter);
+            if (index == -1)
+              {
+                g_error ("Unknown setter %s for property %s:%s", prop->setter, parent->name, node->name);
+              }
+
+            blob->setter = (guint) index;
+          }
+        else
+          blob->setter = ACCESSOR_SENTINEL;
+
+        if (prop->getter != NULL)
+          {
+            int index = get_index_of_member_type ((GIrNodeInterface*)parent,
+                                                  G_IR_NODE_FUNCTION,
+                                                  prop->getter);
+            if (index == -1)
+              {
+                g_error ("Unknown getter %s for property %s:%s", prop->getter, parent->name, node->name);
+              }
+
+            blob->getter = (guint) index;
+          }
+        else
+          blob->getter = ACCESSOR_SENTINEL;
+
         _g_ir_node_build_typelib ((GIrNode *)prop->type,
 				 node, build, offset, offset2, NULL);
       }
@@ -1648,8 +1681,8 @@ _g_ir_node_build_typelib (GIrNode         *node,
 	blob->blob_type = BLOB_TYPE_FUNCTION;
 	blob->deprecated = function->deprecated;
         blob->is_static = !function->is_method;
-	blob->setter = function->is_setter;
-	blob->getter = function->is_getter;
+	blob->setter = FALSE;
+	blob->getter = FALSE;
 	blob->constructor = function->is_constructor;
 	blob->wraps_vfunc = function->wraps_vfunc;
 	blob->throws = function->throws; /* Deprecated. Also stored in SignatureBlob. */
@@ -1658,6 +1691,21 @@ _g_ir_node_build_typelib (GIrNode         *node,
 	blob->symbol = _g_ir_write_string (function->symbol, strings, data, offset2);
 	blob->signature = signature;
 
+        if (function->is_setter || function->is_getter)
+          {
+            int index = get_index_of_member_type ((GIrNodeInterface*)parent,
+                                                  G_IR_NODE_PROPERTY,
+                                                  function->property);
+            if (index == -1)
+              {
+                g_error ("Unknown property %s:%s for accessor %s", parent->name, function->property, function->symbol);
+              }
+
+            blob->setter = function->is_setter;
+            blob->getter = function->is_getter;
+            blob->index = (guint) index;
+          }
+
         /* function->result is special since it doesn't appear in the serialized format but
          * we do want the attributes for it to appear
          */
@@ -2096,6 +2144,7 @@ _g_ir_node_build_typelib (GIrNode         *node,
 	blob->blob_type = BLOB_TYPE_OBJECT;
 	blob->abstract = object->abstract;
         blob->fundamental = object->fundamental;
+        blob->final_ = object->final_;
 	blob->deprecated = object->deprecated;
 	blob->reserved = 0;
 	blob->name = _g_ir_write_string (node->name, strings, data, offset2);
