@@ -103,6 +103,7 @@ struct _GIrNodeFunction
   gboolean instance_transfer_full;
 
   gchar *symbol;
+  char *property;
 
   GIrNodeParam *result;
   GList *parameters;
@@ -173,6 +174,9 @@ struct _GIrNodeProperty
   gboolean transfer;
   gboolean shallow_transfer;
 
+  char *setter;
+  char *getter;
+
   GIrNodeType *type;
 };
 
@@ -240,6 +244,7 @@ struct _GIrNodeInterface
   gboolean abstract;
   gboolean deprecated;
   gboolean fundamental;
+  gboolean final_;
 
   gchar *gtype_name;
   gchar *gtype_init;
