@@ -423,6 +423,7 @@ write_field_info (const gchar *namespace,
   if (branch)
     {
       xml_printf (file, " branch=\"");
+      g_base_info_unref ((GIBaseInfo *)type);
       type = g_constant_info_get_type (branch);
       g_constant_info_get_value (branch, &value);
       write_constant_value (namespace, type, &value, file);
@@ -585,10 +586,22 @@ write_function_info (const gchar    *namespace,
   xml_printf (file, " name=\"%s\" c:identifier=\"%s\"",
               name, symbol);
 
-  if (flags & GI_FUNCTION_IS_SETTER)
-    xml_printf (file, " type=\"setter\"");
-  else if (flags & GI_FUNCTION_IS_GETTER)
-    xml_printf (file, " type=\"getter\"");
+  if ((flags & GI_FUNCTION_IS_SETTER) || (flags & GI_FUNCTION_IS_GETTER))
+    {
+      GIPropertyInfo *property = g_function_info_get_property (info);
+
+      if (property != NULL)
+        {
+          const char *property_name = g_base_info_get_name ((GIBaseInfo *)property);
+
+          if (flags & GI_FUNCTION_IS_SETTER)
+            xml_printf (file, " glib:set-property=\"%s\"", property_name);
+          else if (flags & GI_FUNCTION_IS_GETTER)
+            xml_printf (file, " glib:get-property=\"%s\"", property_name);
+
+          g_base_info_unref (property);
+        }
+    }
 
   if (deprecated)
     xml_printf (file, " deprecated=\"1\"");
@@ -922,7 +935,10 @@ write_vfunc_info (const gchar *namespace,
   xml_printf (file, " offset=\"%d\"", offset);
 
   if (invoker)
-    xml_printf (file, " invoker=\"%s\"", g_base_info_get_name ((GIBaseInfo*)invoker));
+    {
+      xml_printf (file, " invoker=\"%s\"", g_base_info_get_name ((GIBaseInfo*)invoker));
+      g_base_info_unref ((GIBaseInfo *)invoker);
+    }
 
   write_callable_info (namespace, (GICallableInfo*)info, file);
 
@@ -961,6 +977,28 @@ write_property_info (const gchar    *namespace,
   if (flags & G_PARAM_CONSTRUCT_ONLY)
     xml_printf (file, " construct-only=\"1\"");
 
+  if (flags & G_PARAM_READABLE)
+    {
+      GIFunctionInfo *getter = g_property_info_get_getter (info);
+
+      if (getter != NULL)
+        {
+          xml_printf (file, " getter=\"%s\"", g_base_info_get_name ((GIBaseInfo *) getter));
+          g_base_info_unref ((GIBaseInfo *) getter);
+        }
+    }
+
+  if (flags & G_PARAM_WRITABLE)
+    {
+      GIFunctionInfo *setter = g_property_info_get_setter (info);
+
+      if (setter != NULL)
+        {
+          xml_printf (file, " setter=\"%s\"", g_base_info_get_name ((GIBaseInfo *) setter));
+          g_base_info_unref ((GIBaseInfo *) setter);
+        }
+    }
+
   write_ownership_transfer (g_property_info_get_ownership_transfer (info), file);
 
   write_attributes (file, (GIBaseInfo*) info);
@@ -984,6 +1022,7 @@ write_object_info (const gchar  *namespace,
   gboolean deprecated;
   gboolean is_abstract;
   gboolean is_fundamental;
+  gboolean is_final;
   GIObjectInfo *pnode;
   GIStructInfo *class_struct;
   gint i;
@@ -992,6 +1031,7 @@ write_object_info (const gchar  *namespace,
   deprecated = g_base_info_is_deprecated ((GIBaseInfo *)info);
   is_abstract = g_object_info_get_abstract (info);
   is_fundamental = g_object_info_get_fundamental (info);
+  is_final = g_object_info_get_final (info);
 
   type_name = g_registered_type_info_get_type_name ((GIRegisteredTypeInfo*)info);
   type_init = g_registered_type_info_get_type_init ((GIRegisteredTypeInfo*)info);
@@ -1015,6 +1055,9 @@ write_object_info (const gchar  *namespace,
   if (is_abstract)
     xml_printf (file, " abstract=\"1\"");
 
+  if (is_final)
+    xml_printf (file, " final=\"1\"");
+
   xml_printf (file, " glib:type-name=\"%s\" glib:get-type=\"%s\"", type_name, type_init);
 
   if (is_fundamental)
