@@ -903,6 +903,8 @@ typedef struct {
   ValueBlob values[];
 } EnumBlob;
 
+#define ACCESSOR_SENTINEL       0x3ff
+
 /**
  * PropertyBlob:
  * @name: The name of the property.
@@ -918,6 +920,10 @@ typedef struct {
  *   ownership of the container, but not of its contents, is transferred.
  *   This is typically the case when reading lists of statically allocated
  *   things.
+ * @setter: the index of the setter function for this property, if @writable
+ *   is set; if the method is not known, the value will be set to %ACCESSOR_SENTINEL
+ * @getter: ths index of the getter function for this property, if @readable
+ *   is set; if the method is not known, the value will be set to %ACCESSOR_SENTINEL
  * @reserved: Reserved for future use.
  * @reserved2: Reserved for future use.
  * @type: Describes the type of the property.
@@ -934,7 +940,9 @@ typedef struct {
   guint32        construct_only               : 1;
   guint32        transfer_ownership           : 1;
   guint32        transfer_container_ownership : 1;
-  guint32        reserved                     :25;
+  guint32        setter                       :10;
+  guint32        getter                       :10;
+  guint32        reserved                     : 5;
 
   guint32        reserved2;
 
@@ -1034,10 +1042,11 @@ typedef struct {
 /**
  * ObjectBlob:
  * @blob_type: #BLOB_TYPE_OBJECT
- * @deprecated: TODO
- * @abstract: TODO
+ * @deprecated: whether the type is deprecated
+ * @abstract: whether the type can be instantiated
  * @fundamental: this object is not a GObject derived type, instead it's
  *   an additional fundamental type.
+ * @final: whether the type can be derived
  * @reserved: Reserved for future use.
  * @name: TODO
  * @gtype_name: String name of the associated #GType
@@ -1076,7 +1085,8 @@ typedef struct {
   guint16   deprecated   : 1;
   guint16   abstract     : 1;
   guint16   fundamental  : 1;
-  guint16   reserved     :13;
+  guint16   final_       : 1;
+  guint16   reserved     :12;
   guint32   name;
 
   guint32   gtype_name;
