@@ -2139,7 +2139,7 @@ prefix_with_context (GError **error,
 		     const char *section,
 		     ValidateContext *ctx)
 {
-  GString *str = g_string_new (NULL);
+  GString *str;
   GSList *link;
   char *buf;
 
@@ -2150,6 +2150,8 @@ prefix_with_context (GError **error,
       return;
     }
 
+  str = g_string_new (NULL);
+
   for (; link; link = link->next)
     {
       g_string_append (str, link->data);
@@ -2235,7 +2237,7 @@ static GSList *library_paths;
  * path (ie. LD_LIBRARY_PATH and DT_RPATH in ELF systems).
  * See the documentation of your dynamic linker for full details.
  *
- * Since: 1.35.8
+ * Since: 1.36
  */
 void
 g_irepository_prepend_library_path (const char *directory)
