@@ -18,13 +18,7 @@
 # 02110-1301, USA.
 #
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-from __future__ import unicode_literals
-
 import os
-import sys
 import argparse
 
 import giscanner
@@ -32,7 +26,7 @@ from .docwriter import DocWriter
 from .sectionparser import generate_sections_file, write_sections_file
 from .transformer import Transformer
 
-FORMATS = ('mallard', 'sections')
+FORMATS = ('devdocs', 'mallard', 'sections')
 
 
 def doc_main(args):
@@ -49,7 +43,7 @@ def doc_main(args):
                       help="Output language")
     parser.add_argument("-f", "--format",
                         action="store", dest="format",
-                        choices=FORMATS, default=FORMATS[0],
+                        choices=FORMATS, default=FORMATS[1],
                         help="Output format")
     parser.add_argument("-I", "--add-include-path",
                       action="append", dest="include_paths", default=[],
@@ -61,6 +55,9 @@ def doc_main(args):
     args = parser.parse_args(args[1:])
     if not args.output:
         raise SystemExit("missing output parameter")
+    if args.format not in FORMATS:
+        raise SystemExit("Unknown output format %s (supported: %s)" %
+            (args.format, ", ".join(FORMATS)))
 
     if 'UNINSTALLED_INTROSPECTION_SRCDIR' in os.environ:
         top_srcdir = os.environ['UNINSTALLED_INTROSPECTION_SRCDIR']
@@ -74,7 +71,7 @@ def doc_main(args):
     if args.format == 'sections':
         sections_file = generate_sections_file(transformer)
 
-        with open(args.output, 'w') as fp:
+        with open(args.output, 'w', encoding='utf-8') as fp:
             write_sections_file(fp, sections_file)
     else:
         writer = DocWriter(transformer, args.language, args.format)
