@@ -138,6 +138,7 @@ class GIRParser(object):
             _corens('interface'): self._parse_object_interface,
             _corens('record'): self._parse_record,
             _corens('union'): self._parse_union,
+            _corens('docsection'): self._parse_doc_section,
             _glibns('boxed'): self._parse_boxed}
 
         if not self._types_only:
@@ -150,6 +151,11 @@ class GIRParser(object):
             if method is not None:
                 method(node)
 
+    def _parse_doc_section(self, node):
+        docsection = ast.DocSection(node.attrib["name"])
+        self._parse_generic_attribs(node, docsection)
+        self._namespace.append(docsection)
+
     def _parse_include(self, node):
         include = ast.Include(node.attrib['name'], node.attrib['version'])
         self._includes.add(include)
@@ -253,6 +259,9 @@ class GIRParser(object):
             is_abstract = node.attrib.get('abstract')
             is_abstract = is_abstract and is_abstract != '0'
             ctor_kwargs['is_abstract'] = is_abstract
+            is_final = node.attrib.get('final')
+            is_final = is_final and is_final != '0'
+            ctor_kwargs['is_final'] = is_final
         else:
             raise AssertionError(node)
 
@@ -373,6 +382,8 @@ class GIRParser(object):
         else:
             assert False
 
+        func.set_property = node.attrib.get(_glibns('set-property'), None)
+        func.get_property = node.attrib.get(_glibns('get-property'), None)
         func.shadows = node.attrib.get('shadows', None)
         func.shadowed_by = node.attrib.get('shadowed-by', None)
         func.moved_to = node.attrib.get('moved-to', None)
@@ -595,15 +606,18 @@ class GIRParser(object):
                             node.attrib.get('construct') == '1',
                             node.attrib.get('construct-only') == '1',
                             node.attrib.get('transfer-ownership'))
-        self._parse_generic_attribs(node, prop)
+        prop.setter = node.attrib.get('setter')
+        prop.getter = node.attrib.get('getter')
         prop.parent = parent
+        self._parse_generic_attribs(node, prop)
         return prop
 
     def _parse_member(self, node):
         member = ast.Member(node.attrib['name'],
                             node.attrib['value'],
                             node.attrib.get(_cns('identifier')),
-                            node.attrib.get(_glibns('nick')))
+                            node.attrib.get(_glibns('nick')),
+                            node.attrib.get(_glibns('name')))
         self._parse_generic_attribs(node, member)
         return member
 
