@@ -24,10 +24,11 @@ from . import message
 from .annotationparser import (TAG_DEPRECATED, TAG_SINCE, TAG_STABILITY, TAG_RETURNS)
 from .annotationparser import (ANN_ALLOW_NONE, ANN_ARRAY, ANN_ATTRIBUTES, ANN_CLOSURE,
                                ANN_CONSTRUCTOR, ANN_DESTROY, ANN_ELEMENT_TYPE, ANN_FOREIGN,
-                               ANN_GET_VALUE_FUNC, ANN_IN, ANN_INOUT, ANN_METHOD, ANN_OUT,
-                               ANN_REF_FUNC, ANN_RENAME_TO, ANN_SCOPE, ANN_SET_VALUE_FUNC,
-                               ANN_SKIP, ANN_TRANSFER, ANN_TYPE, ANN_UNREF_FUNC, ANN_VALUE,
-                               ANN_VFUNC, ANN_NULLABLE, ANN_OPTIONAL, ANN_NOT)
+                               ANN_GET_PROPERTY, ANN_GET_VALUE_FUNC, ANN_GETTER, ANN_IN, ANN_INOUT,
+                               ANN_METHOD, ANN_OUT, ANN_REF_FUNC, ANN_RENAME_TO, ANN_SCOPE,
+                               ANN_SET_PROPERTY, ANN_SET_VALUE_FUNC, ANN_SETTER, ANN_SKIP, ANN_TRANSFER,
+                               ANN_TYPE, ANN_UNREF_FUNC, ANN_VALUE, ANN_VFUNC, ANN_NULLABLE,
+                               ANN_OPTIONAL, ANN_NOT)
 from .annotationparser import (OPT_ARRAY_FIXED_SIZE, OPT_ARRAY_LENGTH, OPT_ARRAY_ZERO_TERMINATED,
                                OPT_OUT_CALLEE_ALLOCATES, OPT_OUT_CALLER_ALLOCATES,
                                OPT_TRANSFER_CONTAINER, OPT_TRANSFER_FLOATING, OPT_TRANSFER_NONE)
@@ -69,6 +70,10 @@ class MainTransformer(object):
         # Read in most annotations now.
         self._namespace.walk(self._pass_read_annotations)
 
+        # Now that we have associated doc SECTIONs to classes,
+        # add the unused section blocks as standalone nodes.
+        self._add_standalone_doc_sections()
+
         # Now that we've possibly seen more types from annotations,
         # do another type resolution pass.
         self._namespace.walk(self._pass_type_resolution)
@@ -87,6 +92,7 @@ class MainTransformer(object):
                 self._pair_function(node)
             if isinstance(node, (ast.Class, ast.Interface)):
                 self._pair_class_virtuals(node)
+                self._pair_property_accessors(node)
 
         # Some annotations need to be post function pairing
         self._namespace.walk(self._pass_read_annotations2)
@@ -101,6 +107,14 @@ class MainTransformer(object):
 
     # Private
 
+    def _add_standalone_doc_sections(self):
+        for block_name, block in self._blocks.items():
+            if block_name.startswith("SECTION:") and block.description:
+                node = ast.DocSection(block_name[8:])
+                node.doc = block.description
+                node.doc_position = block.position
+                self._namespace.append(node)
+
     def _pass_fixup_hidden_fields(self, node, chain):
         """Hide all callbacks starting with _; the typical
         usage is void (*_gtk_reserved1)(void);"""
@@ -240,7 +254,9 @@ class MainTransformer(object):
                 self._apply_annotations_field(node, block, field)
             name = self._get_annotation_name(node)
             section_name = 'SECTION:%s' % (name.lower(), )
-            block = self._blocks.get(section_name)
+            # We pop it from our blocks so that we can serialize leftover
+            # SECTIONs as standalone nodes
+            block = self._blocks.pop(section_name, None)
             self._apply_annotations_annotated(node, block)
         if isinstance(node, (ast.Class, ast.Interface)):
             for prop in node.properties:
@@ -759,6 +775,14 @@ class MainTransformer(object):
         if ANN_METHOD in block.annotations:
             node.is_method = True
 
+        set_property = block.annotations.get(ANN_SET_PROPERTY)
+        if set_property is not None and isinstance(node, ast.Function):
+            node.set_property = set_property[0]
+
+        get_property = block.annotations.get(ANN_GET_PROPERTY)
+        if get_property is not None and isinstance(node, ast.Function):
+            node.get_property = get_property[0]
+
     def _apply_annotations_alias(self, node, chain):
         block = self._get_block(node)
         self._apply_annotations_annotated(node, block)
@@ -858,19 +882,28 @@ class MainTransformer(object):
         self._apply_annotations_params(node, node.parameters, block)
         self._apply_annotations_return(node, node.retval, block)
 
-    def _apply_annotations_field(self, parent, block, field):
-        if not block:
+    def _apply_annotations_field(self, parent, parent_block, field):
+        block = self._blocks.get('%s.%s' % (self._get_annotation_name(parent), field.name))
+
+        # Prioritize block level documentation
+        if block:
+            self._apply_annotations_annotated(field, block)
+            annotations = block.annotations
+        elif not parent_block:
             return
-        tag = block.params.get(field.name)
-        if not tag:
-            return
-        type_annotation = tag.annotations.get(ANN_TYPE)
+        else:
+            tag = parent_block.params.get(field.name)
+            if not tag:
+                return
+            annotations = tag.annotations
+            field.doc = tag.description
+            field.doc_position = tag.position
+
+        type_annotation = annotations.get(ANN_TYPE)
         if type_annotation:
             field.type = self._transformer.create_type_from_user_string(type_annotation[0])
-        field.doc = tag.description
-        field.doc_position = tag.position
         try:
-            self._adjust_container_type(parent, field, tag.annotations)
+            self._adjust_container_type(parent, field, annotations)
         except AttributeError as ex:
             print(ex)
 
@@ -891,6 +924,12 @@ class MainTransformer(object):
         type_annotation = block.annotations.get(ANN_TYPE)
         if type_annotation:
             prop.type = self._resolve_toplevel(type_annotation[0], prop.type, prop, parent)
+        setter = block.annotations.get(ANN_SETTER)
+        if setter:
+            prop.setter = setter[0]
+        getter = block.annotations.get(ANN_GETTER)
+        if getter:
+            prop.getter = getter[0]
 
     def _apply_annotations_signal(self, parent, signal):
         names = []
@@ -938,15 +977,21 @@ class MainTransformer(object):
         if value_annotation:
             node.value = value_annotation[0]
 
-    def _apply_annotations_enum_members(self, node, block):
-        if block is None:
-            return
-
+    def _apply_annotations_enum_members(self, node, parent_block):
         for m in node.members:
-            param = block.params.get(m.symbol, None)
-            if param and param.description:
-                m.doc = param.description
-                m.doc_position = param.position
+            block = self._blocks.get(m.symbol)
+            # Prioritize block-level documentation
+            if block:
+                self._apply_annotations_annotated(m, block)
+            elif parent_block:
+                param = parent_block.params.get(m.symbol)
+
+                if not param:
+                    continue
+
+                if param.description:
+                    m.doc = param.description
+                    m.doc_position = param.position
 
     def _pass_read_annotations2(self, node, chain):
         if isinstance(node, ast.Function):
@@ -1259,10 +1304,21 @@ method or constructor of some type."""
         return origin_node
 
     def _get_constructor_name(self, func, subsymbol):
-        name = None
+        prefix_matches = False
+        uscored_prefix = None
+        target = self._transformer.lookup_typenode(func.retval.type)
+        if hasattr(target, 'c_symbol_prefix') and target.c_symbol_prefix is not None:
+            prefix_matches = subsymbol.startswith(target.c_symbol_prefix)
+            if prefix_matches:
+                uscored_prefix = target.c_symbol_prefix
+        if not prefix_matches:
+            uscored_prefix = self._uscored_identifier_for_type(func.retval.type)
         split = self._split_uscored_by_type(subsymbol)
         if split is None:
             if func.is_constructor:
+                if uscored_prefix in func.symbol:
+                    subsym_idx = func.symbol.find(subsymbol)
+                    func.name = func.symbol[(subsym_idx + len(uscored_prefix) + 1):]
                 name = func.name
         else:
             _, name = split
@@ -1418,6 +1474,56 @@ method or constructor of some type."""
                 self._apply_annotations_callable(vfunc, [], block)
                 break
 
+    def _pair_property_accessors(self, node):
+        """Look for accessor methods for class properties"""
+        for prop in node.properties:
+            if not prop.introspectable:
+                continue
+            if prop.setter is None:
+                normalized_name = prop.name.replace('-', '_')
+                if prop.writable and not prop.construct_only:
+                    setter = 'set_' + normalized_name
+                else:
+                    setter = None
+            else:
+                setter = prop.setter
+            if prop.getter is None:
+                if prop.readable:
+                    getter = ['get_' + normalized_name]
+                    # Heuristic: read-only properties can have getters that are
+                    # just the property name, like: gtk_widget_has_focus()
+                    if not prop.writable and prop.type.is_equiv(ast.TYPE_BOOLEAN):
+                        getter.append(normalized_name)
+                else:
+                    getter = []
+            else:
+                getter = [prop.getter]
+            for method in node.methods:
+                if not method.introspectable:
+                    continue
+                if setter is not None and method.name == setter:
+                    if method.set_property is None:
+                        method.set_property = prop.name
+                    elif method.set_property != prop.name:
+                        message.warn_node(method,
+                                          "Setter method '%s' for property '%s' has a "
+                                          "mismatched '(set-property %s)' annotation" %
+                                          (method.symbol, prop.name, method.set_property))
+                        method.set_property = prop.name
+                    prop.setter = method.name
+                    continue
+                if getter is not [] and method.name in getter:
+                    if method.get_property is None:
+                        method.get_property = prop.name
+                    elif method.get_property != prop.name:
+                        message.warn_node(method,
+                                          "Getter method '%s' for property '%s' has a "
+                                          "mismatched '(get-property %s)' annotation" %
+                                          (method.symbol, prop.name, method.get_property))
+                        method.get_property = prop.name
+                    prop.getter = method.name
+                    continue
+
     def _pass3(self, node, chain):
         """Pass 3 is after we've loaded GType data and performed type
         closure."""
